#ifndef OCC_CONNECT_H
#define OCC_CONNECT_H

#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <deque>
#include <map>
#include <vector>
#include <set>

/* OpenCASCADE ************************************************************/
#include <config.h>

#include <TopoDS_Shape.hxx>
#include <TopoDS_Edge.hxx>
#include <TopoDS_Wire.hxx>
#include <TopoDS_Face.hxx>

#include <BRep_Builder.hxx>
#include <LocOpe_SplitShape.hxx>

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
class OCC_Connect {
    struct LessThanIntegerSet {
	bool operator()(std::set<int> const &a, std::set<int> const &b);
    };
    typedef std::map<std::set<int>,std::set<int>,LessThanIntegerSet> mapping_t;

    struct FaceCutters:std::vector<TopoDS_Wire> {
	void Add(TopoDS_Wire const&);
    };

    std::deque<TopoDS_Shape> assembly;
protected:
    int verbose;

public:
    enum Verbose { Cutting=1, CuttingReject=4 };
    OCC_Connect(int v=0) { verbose=v; }
    int SaveBRep(char const *);
    void Connect(void);
    void Collect(void);
    void Add(TopoDS_Shape const &a) { assembly.push_back(a); }
    void Dump(std::ostream &) const;
    operator TopoDS_Shape(void) { return assembly.front(); }
private:
    void MergeVertices(TopoDS_Shape&,TopoDS_Shape&) const;
    void MergeEdges(TopoDS_Shape&,TopoDS_Shape&) const;
    void MergeFaces(TopoDS_Shape&) const;
    void PerformBoolean(union tree*);
    void Intersect(BRep_Builder &BB, TopoDS_Shape &target, 
        TopoDS_Shape &shape, TopoDS_Shape &tool);
    bool CanMergeCurve(TopoDS_Edge edge1,TopoDS_Edge edge2) const;
    bool CanMergeFace(TopoDS_Face face1,TopoDS_Face face2) const;
    void PrintItemCount(TopoDS_Shape const &shape) const;
};

#endif
